// Updates: 2004.07.29


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.util.regex.*;


/**
 * A program to rename files using the power of regular expressions.
 * @version 0.1
 * @author Michel Deriaz
 */
public class RenameFiles extends JFrame implements ActionListener, DropTargetListener {
  private File dir; // the directory
  private String[] oldFiles, newFiles;
  private int maxLength = -1; // length of the biggest filename (old files)
  private StyledTextZone stz;
  private JTextFieldX tfSearch, tfReplace;
  private JCheckBox cbLive;
  private JButton butTest, butReplace, butOpen;
 
  
  /**
   * This constructor creates the GUI.
   */  
  public RenameFiles() {
    super("Rename Files");
    setIconImage(new ImageIcon(getClass().getResource("icon.png")).getImage());
    JPanel p = new JPanel(new BorderLayout());
    p.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    
    stz = new StyledTextZone("Files", 0);
    stz.setEditable(false);
    stz.setFont(-1, "Monospaced");
    p.add(stz, BorderLayout.CENTER);
    
    JPanel panSouth = new JPanel(new GridLayout(4, 1));

    JPanel panSearch = new JPanel();
    JLabel labSearch = new JLabel("Search (regex):");
    int height = labSearch.getPreferredSize().height;
    labSearch.setPreferredSize(new Dimension(115, height));    	
    panSearch.add(labSearch);
    tfSearch = new JTextFieldX(this, "Search.obj", 25);
    panSearch.add(tfSearch);
    panSouth.add(panSearch);

    JPanel panReplace = new JPanel();
    JLabel labReplace = new JLabel("Replace by (regex):");
    labReplace.setPreferredSize(new Dimension(115, height));    	
    panReplace.add(labReplace);
    tfReplace = new JTextFieldX(this, "Replace.obj", 25);
    panReplace.add(tfReplace);
    panSouth.add(panReplace);

    panSouth.add(new Container());

    JPanel panCmd = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 0));
    cbLive = new JCheckBox("Live eval.", true);
    cbLive.addActionListener(this);
    panCmd.add(cbLive);
    butTest = new JButton("Test");
    butTest.setEnabled(false);
    butTest.addActionListener(this);
    panCmd.add(butTest);    
    butReplace = new JButton("Replace");
    butReplace.addActionListener(this);
    panCmd.add(butReplace);    
    butOpen = new JButton("Open");
    butOpen.addActionListener(this);
    panCmd.add(butOpen); 
    panSouth.add(panCmd);
    
    p.add(panSouth, BorderLayout.SOUTH);
    
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(480, 640);
    setLocationRelativeTo(null);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    new DropTarget(stz.getJTextPane(), this);
    show();
    help();
  }


  /**
   * Use this method to launch the program from the console.
   */
  public static void main(String[] args) {
    new RenameFiles();
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(cbLive)) {
      butTest.setEnabled(!cbLive.isSelected());
      stz.saveScrollBarPos();
      update();
      stz.restoreScrollBarPos();
    }
    if (e.getSource().equals(butTest)) {
      stz.saveScrollBarPos();
      update();
      stz.restoreScrollBarPos();
    }
    if (e.getSource().equals(butReplace)) replace();    
    if (e.getSource().equals(butOpen)) open();
  }
  
  
  public void drop(DropTargetDropEvent event) {
    event.acceptDrop(DnDConstants.ACTION_COPY);
    Transferable transferable = event.getTransferable();
    DataFlavor[] flavors = transferable.getTransferDataFlavors();
    for (int i = 0; i < flavors.length; i++) { 
      DataFlavor df = flavors[i];
      try {
        if (df.equals(DataFlavor.javaFileListFlavor)) {
          java.util.List fileList = (java.util.List)transferable.getTransferData(df);
          if (fileList == null) return;
          File f = (File)fileList.get(0);
          if (!f.isDirectory()) return;
          oldFiles = null;
          dir = f;
          getFilenames();
        }
      }
      catch(Exception e) { 
        System.out.println(e);
      }
    }
    event.dropComplete(true);
  }


  /**
   * Called by the JTextField class.
   */  
  protected void tfChanged(JTextField src) {
    if (!cbLive.isSelected()) return;
    stz.saveScrollBarPos();
    update();
    stz.restoreScrollBarPos();
  }


  public void dragEnter(DropTargetDragEvent event) {}
  public void dragExit(DropTargetEvent event) {}
  public void dragOver(DropTargetDragEvent event) {}
  public void dropActionChanged(DropTargetDragEvent event) {}  
    

  private void replace() {
    update();
    if (oldFiles == null || newFiles == null) return;
    ArrayList list = new ArrayList();
    for (int i = 0; i < newFiles.length; i++) {
      if (newFiles[i].equals("")) {
        JOptionPane.showMessageDialog(this, "At least one destination file has no name!", "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }        
      if (list.contains(newFiles[i])) {
        JOptionPane.showMessageDialog(this, "Several destination files have the same name!", "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      list.add(newFiles[i]);
    }
    for (int i = 0; i < oldFiles.length; i++) {
      File fOld = new File(dir + "/" + oldFiles[i]);
      File fNew = new File(dir + "/" + newFiles[i]);
      fOld.renameTo(fNew);
    }
    oldFiles = null;
    newFiles = null;
    setTitle("Rename Files");
    stz.setText("Done.");
  }

  
  private void open() {
    JFileChooser chooser = new JFileChooser(new File("."));
    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);  
    chooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
      public boolean accept(File f) {
        return true;
      }
      public String getDescription() {
        return "Directories";
      }
    });    
    int returnVal = chooser.showOpenDialog(this);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      oldFiles = null;
      dir = chooser.getSelectedFile();
      getFilenames();
    }
  }


  private void getFilenames() {
    setTitle("Rename Files   -   " + dir);
    File[] listFiles = dir.listFiles(new FileFilter() {
      public boolean accept(File pathname) {
        return !pathname.isDirectory();
      }
    });
    if (listFiles == null) return;
    oldFiles = new String[listFiles.length];
    maxLength = -1;
    for (int i = 0; i < listFiles.length; i++) {
      oldFiles[i] = listFiles[i].getName();
      if (oldFiles[i].length() > maxLength) maxLength = oldFiles[i].length();
    }
    update();
  }
          
 
  private void update() {
    try {
      stz.setText("");
      if (oldFiles == null) return;
      newFiles = new String[oldFiles.length];
      String regex = tfSearch.getText();
      Pattern p = Pattern.compile(regex);
      String replace = tfReplace.getText();
      for (int i = 0; i < oldFiles.length; i++) {
        String of = oldFiles[i];
        Matcher m = p.matcher(of);
        if (m.find()) {
          int start = m.start();
          int end = m.end();
          newFiles[i] = of.replaceFirst(regex, replace);
          stz.print(of.substring(0, start));
          stz.print(of.substring(start, end), StyledTextZone.RED);
          stz.print(of.substring(end));
        }
        else {
          newFiles[i] = oldFiles[i];
          stz.print(oldFiles[i]);
        }
        for (int j = 0; j < maxLength - of.length(); j++) {
          stz.print(" ");
        }
        stz.println(" > " + newFiles[i]);
      }
    }
    catch(PatternSyntaxException e) {
      stz.setText("Error in regular expression.");
    }
    catch(StringIndexOutOfBoundsException e) {
      stz.setText("Error in regular expression.");
    }
  }
  
  
  private void help() {
    String helpText = "\n- You can also open a directory with a drag-and-drop operation"
        + "\nover this text zone."
        + "\n\n- Use regular expressions to search specific patterns in the"
        + "\nfilenames. More information about regular expressions can be"
        + "\nfound at http://cui.unige.ch/~deriazm/Softs/RegexSR/"
        + "\n\n- To record a regular expression, press the enter key; to"
        + "\nrestore previous expressions, use the up and down arrows."
        + "\n\nExample 1: Renaming the following files:"
        + "\n   DSCN1989.JPG > DSCN1989Modified.JPG"
        + "\n   DSCN1990.JPG > DSCN1990Modified.JPG"
        + "\n   DSCN1991.JPG > DSCN1991Modified.JPG"
        + "\nYou will use Search = \"\\.\" and Replace = \"Modified\\.\""
        + "\n\nExample 2: Renaming only JPG files:"
        + "\n   DSCN1989.JPG > DSCN_1989.JPG"
        + "\n   DSCN1990.JPG > DSCN_1990.JPG"
        + "\n   DSCN1991.JPG > DSCN_1991.JPG"
        + "\n   DSCN1995.PNG > DSCN1995.PNG"
        + "\n   DSCN1996.PNG > DSCN1996.PNG"
        + "\nYou will use Search = \"(?i)(\\d{4}\\.jpg)$\" and Replace = \"_$1\"";
    stz.setText(helpText);    
    stz.setStyle(StyledTextZone.BLUE);
  }
}